#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>

#include "adc.h"
#include "macros.h"
#include "pindefs.h"

volatile int pwm = 0;
volatile int led = 0;
volatile unsigned char RED[] = {0, 0, 0, 0};
volatile unsigned char GREEN[] = {0, 0, 0, 0};
volatile unsigned char BLUE[] = {0, 0, 0, 0};

const unsigned char colors[][3] =
{
//    {0, 15, 0},
//    {15, 0, 0},
    {0, 0, 15},
    {15, 15, 0},
    {15, 0, 15},
    {0, 15, 15},
    {15, 15, 15}
};

int button(int id)
{
    switch(id) {
        case 1:
            return !(PINB & BUTTON1) ? 1 : 0;
        case 2:
            return !(PINB & BUTTON2) ? 1 : 0;
        case 3:
            return !(PINB & BUTTON3) ? 1 : 0;
        case 4:
            return !(PINA & BUTTON4) ? 1 : 0;
        default:
            return 0;
    }
}

void play_pattern(int seed, int level)
{
    srand(seed);
    int i;
    
    for (i = 0; i < level; ++i) {
        int led = rand() % 4;
        int c = rand() % 5;
        RED[led] = colors[c][0];
        GREEN[led] = colors[c][1];
        BLUE[led] = colors[c][2];
        
        _delay_ms(150);
        RED[led] = 0;
        GREEN[led] = 0;
        BLUE[led] = 0;
        
        if (i < level-1)
            _delay_ms(150);
    }
}

unsigned char listen(int seed, int level)
{
    srand(seed);
    int i;
    
    for (i = 0; i < level; ++i) {
        int target = rand() % 4;
        int c = rand() % 5;
        
        int b1, b2, b3, b4;
        
        do {
            b1 = button(1);
            b2 = button(2);
            b3 = button(3);
            b4 = button(4);
        } while (!b1 && !b2 && !b3 && !b4);
        
        // if they were holding down multiple buttons, then fail.
        if (b1 + b2 + b3 + b4 > 1)
            return 0; 

        if (target == 0 && !b1)
            return 0;
        if (target == 1 && !b2)
            return 0;
        if (target == 2 && !b3)
            return 0;
        if (target == 3 && !b4)
            return 0;

        RED[target] = colors[c][0];
        GREEN[target] = colors[c][1];
        BLUE[target] = colors[c][2];
        
        while(button(1) || button(2) || button(3) || button(4))
            ;
    
        RED[target] = 0;
        GREEN[target] = 0;
        BLUE[target] = 0;
        
    }
    return 1;
}

void flash_red()
{
    int i;
    int j;
    
    for (j = 0; j < 10; ++j) {
        for (i = 0; i < 4; ++i) {
            RED[i] = 15;
            BLUE[i] = 0;
            GREEN[i] = 0;
        }
        _delay_ms(25);
        for (i = 0; i < 4; ++i)
            RED[i] = 0;
        _delay_ms(25);
    }
}

void flash_green(int level)
{
    int i;
    int j;
    _delay_ms(100);
    for (j = 0; j < level; ++j) {
        for (i = 0; i < 4; ++i) {
            RED[i] = 0;
            BLUE[i] = 0;
            GREEN[i] = 15;
        }
        _delay_ms(25);
        for (i = 0; i < 4; ++i)
            GREEN[i] = 0;
        _delay_ms(25);
    }
}


int main(void)
{
    adc_init(7);
    int i, seed = 0;
    for (i = 0; i < 10; ++i)
        seed += adc_read();
    srandom(seed);

    PORTA |= OFF1 | OFF2 | OFF3 | OFF4 | BUTTON4;
    PORTB |= BUTTON1 | BUTTON2 | BUTTON3;
    DDRA  |= OFF1 | OFF2 | OFF3 | OFF4 | R | G | B;
    
    TCCR0B |= (1<<CS00);
    TIMSK0 |= (1<<TOIE0);

    while(!button(1) && !button(2) && !button(3) && !button(4))
        ;

    while(button(1) || button(2) || button(3) || button(4))
        ;

    sei();
    int level = 1;
    while(1) {
        _delay_ms(100);
        play_pattern(seed, level);
        if (!listen(seed, level)) {
            flash_red();
            level = 1;
            PORTA &= ~BUTTON4; // turn off pull-up
            
            seed += adc_read();
            PORTA |= BUTTON4; // turn on pull-up
        } else {
            flash_green(level);
            ++level;
        }
    }
    return 0;
}

ISR(TIM0_OVF_vect)
{
    if (++led == 4) {
        if (++pwm == 16)
            pwm = 0;
        led = 0;
    }
    
    PORTA &= ~(R|G|B);
    PORTA |= OFF1 | OFF2 | OFF3 | OFF4;
    
    // Turn on appropriate LEDs
    if (pwm < RED[led])
        set(PORTA, R);
    if (pwm < GREEN[led])
        set(PORTA, G);
    if (pwm < BLUE[led])
        set(PORTA, B);
    
    // Activate appropriate ports
    if (led == 0)
        PORTA &= ~(OFF1);
    else if (led == 1)
        PORTA &= ~(OFF2);
    else if (led == 2)
        PORTA &= ~(OFF3);
    else if (led == 3)
        PORTA &= ~(OFF4);
}